;	music cycle

			processor 6502
			include vcs.h
	
			seg.u vars
			org $80
	
frameCnt	ds 2
prevSWB		ds 1

curStartCol	ds 1

; a song is a sequence of measures, which is a sequence of notes
cursong		ds 1	; current song

v0jiff		ds 1	; frame delay count
v0mpos		ds 1	; measure position
v0mptr		ds 2	; measure pointer
v0spos		ds 1	; song position -- $80:$FF off; $40-$7F new song
v0sptr		ds 2	; song pointer

v1jiff		ds 1	; frame delay count
v1mpos		ds 1	; measure position
v1mptr		ds 2	; measure pointer
v1spos		ds 1	; song position -- $80:$FF off; $40-$7F new song
v1sptr		ds 2	; song pointer

			seg code
			org $F000
	
INIT
		SEI
		CLD
		LDA #$00
		TAX
		DEX
		TXS
INIT.1
		STA $00,X
		DEX
		BNE INIT.1
		
; one-time initialization

; ***** MORE BITS *****
		LDA #$34	; three more bits of RAM here
		STA SWBCNT
; *****           *****
		
		LDA #$03	; D0 reflect, D1 score color, D2 PF over P0
		STA CTRLPF	; D5 D4 ball size
		
; per-reset initialization

RESET
		LDA #$00
		STA curStartCol
		
		LDA #$40
		STA cursong
		
		LDA #$FF		; no sound
		STA v0spos

; generate VBLANK, do VSYNC

TOPSCAN
		LDA #$82	; turn on VBLANK, discharge paddles
		STA VBLANK
		LDA #$02	; enable VSYNC
		STA WSYNC
		STA VSYNC
		STA WSYNC	; waiting
		STA WSYNC	; waiting
		LDA #$2C		; set up timer for end of VBLANK
		STA TIM64T	
		INC frameCnt	; keep up with frame count
		BNE TOPSCAN.1
		INC frameCnt+1
TOPSCAN.1
		LDA #$00	
		STA WSYNC	; waiting
		STA VSYNC	; disable VSYNC
		LDA #$02	; charge paddles, keep VBLANK on
		STA VBLANK
		
VBLANKPROC

SW_CHECK
		LDA SWCHB	; state of switches
		AND #$03	; only select and reset
		CMP prevSWB	; match previous value?
		BEQ SW_SKIP	; ignore if matches previous
		STA prevSWB	; store new value
		ROR			; nifty trick, rotate reset bit into carry flag
		BCC SW_RESET
		ROR			; nifty trick, rotate select bit into carry flag
		BCS SW_SKIP
SW_SELE
		LDA curStartCol	; select new variation
		ADC #$10
		STA curStartCol

		LDA cursong
		ADC #$02
		CMP #$44
		BCC SEL.1
		LDA #$40		
SEL.1	STA v0spos		; start song #1
		STA v1spos
		STA cursong

		JMP SW_SKIP
SW_RESET
		LDA #$C0		
		STA v0spos
		
		LDA #$00		; reset current game
		STA frameCnt
		INC frameCnt+1
SW_SKIP

; sound processor -- attack, decay, sustain, release
; vol up, vol down, timbre change, note change
; command, delay, command, delay, command
; volume, note, timbre; delay; etc.

SND_CHECK
V0_START
		BIT v0spos
V0_CHKNONE
		BPL	V0_CHKNEW	; $80:$FF  no song
		BVC V0_CHKFINI
		LDA #$00		; turn off sound
		STA AUDV0
		STA AUDF0
		STA AUDC0
		LDA #$80
		STA v0spos
V0_CHKFINI
		JMP V0_FINISH
V0_CHKNEW
		BVC	V0_NOTETIME	; $40:$7F  new song
V0_NEWSONG
		LDA v0spos		; max $20 songs
		AND #$3F
		TAX
		LDA S_LIST,X
		STA v0sptr
		INX
		LDA S_LIST,X
		STA v0sptr+1
		LDA #$00
		STA v0jiff
		STA v0spos
		JMP V0_NEXTMEAS
		
V0_NOTETIME
		DEC v0jiff
		BPL V0_FINISH	; not time for next command
		LDY v0mpos		; current position in current measure
V0_NEXTNOTE
		INY
		LDA (v0mptr),Y
		TAX				; keep a copy of command in X
		BMI V0_MCMDLY	; $80:$FF a command or delay
V0_FREQVOL
		AND #$40
		BNE V0_FREQ		; $40:$7F a frequency
V0_VOL
		; if LDA (),X would work, no need to save Y
		STY v0mpos		; temp store
		TXA
		STA AUDV0		; volume in low nybble
		AND #$30		; now to look up waveform
		LSR
		LSR
		LSR
		LSR
		TAY
		LDA (v0mptr),Y	; waveform 0,1,2, or 3
		STA AUDC0
		LDY v0mpos
		BNE V0_NEXTNOTE	; next note command
V0_FREQ
		STX AUDF0		; that was easy!
		BPL V0_NEXTNOTE	; next note command
V0_MCMDLY
		AND #$40
		BEQ V0_DELAY
V0_COMMAND
		TXA
		AND #$F0
		CMP #$F0
		BNE V0_NEXTNOTE	; not valid command, get next command
V0_CMDFX
		LDA #$00
		STA AUDC0
		STA AUDF0
		STA AUDV0
		TXA
		AND #$0F
		BPL V0_DLYS
V0_CMDF0
		; additional command
V0_DELAY
		TXA
		AND #$7F		; max delay 127 frames
V0_DLYS	STA v0jiff
		STY v0mpos
		BNE V0_FINISH
V0_NEXTMEAS
		LDY v0spos		; song position
		LDA (v0sptr),Y
		STA v0mptr
		INY
		LDA (v0sptr),Y
		STA v0mptr+1
		INY
		ORA v0mptr		; is the pointer equal to zero?
		BEQ V0_SONGFINI	; nope, keep playing
		CMP #$80		; or equal to $80?
		BNE V0_SONGCONT	; loop this song
V0_SONGLOOP
		LDA #$00
		STA v0spos
		BEQ V0_NEXTMEAS
V0_SONGFINI		
		LDA #$80		; turn song off
		STA v0spos
		JMP V0_FINISH
V0_SONGCONT
		STY v0spos
		LDA #$03
		STA v0mpos
V0_FINISH		

V1_START
		JMP V1_FINISH	; skip second sound
		BIT v1spos
V1_CHKNONE
		BPL	V1_CHKNEW	; $80:$FF  no song
		BVC V1_CHKFINI
		LDA #$00		; turn off sound
		STA AUDV1
		STA AUDF1
		STA AUDC1
		LDA #$80
		STA v1spos
V1_CHKFINI
		JMP V1_FINISH
V1_CHKNEW
		BVC	V1_NOTETIME	; $40:$7F  new song
V1_NEWSONG
		LDA v1spos		; max $20 songs
		AND #$3F
		TAX
		LDA S_LIST,X
		STA v1sptr
		INX
		LDA S_LIST,X
		STA v1sptr+1
		LDA #$00
		STA v1jiff
		STA v1spos
		JMP V1_NEXTMEAS
		
V1_NOTETIME
		DEC v1jiff
		BPL V1_FINISH	; not time for next command
		LDY v1mpos		; current position in current measure
V1_NEXTNOTE
		INY
		LDA (v1mptr),Y
		TAX				; keep a copy of command in X
		BMI V1_MCMDLY	; $80:$FF a command or delay
V1_FREQVOL
		AND #$40
		BNE V1_FREQ		; $40:$7F a frequency
V1_VOL
		; if LDA (),X would work, no need to save Y
		STY v1mpos		; temp store
		TXA
		STA AUDV1		; volume in low nybble
		AND #$30		; now to look up waveform
		LSR
		LSR
		LSR
		LSR
		TAY
		LDA (v1mptr),Y	; waveform 0,1,2, or 3
		STA AUDC1
		LDY v1mpos
		BNE V1_NEXTNOTE	; next note command
V1_FREQ
		STX AUDF1		; that was easy!
		BPL V1_NEXTNOTE	; next note command
V1_MCMDLY
		AND #$40
		BEQ V1_DELAY
V1_COMMAND
		TXA
		AND #$F0
		CMP #$F0
		BNE V1_NEXTNOTE	; not valid command, get next command
V1_CMDFX
		LDA #$00
		STA AUDC1
		STA AUDF1
		STA AUDV1
		TXA
		AND #$0F
		BPL V1_DLYS
V1_CMDF0
		; additional command
V1_DELAY
		TXA
		AND #$7F		; max delay 127 frames
V1_DLYS	STA v1jiff
		STY v1mpos
		BNE V1_FINISH
V1_NEXTMEAS
		LDY v1spos		; song position
		LDA (v1sptr),Y
		STA v1mptr
		INY
		LDA (v1sptr),Y
		STA v1mptr+1
		INY
		ORA v1mptr		; is the pointer equal to zero?
		BEQ V1_SONGFINI	; nope, keep playing
		CMP #$80		; or equal to $80?
		BNE V1_SONGCONT	; loop this song
V1_SONGLOOP
		LDA #$00
		STA v1spos
		BEQ V1_NEXTMEAS
V1_SONGFINI		
		LDA #$80		; turn song off
		STA v1spos
		JMP V1_FINISH
V1_SONGCONT
		STY v1spos
		LDA #$03
		STA v1mpos
V1_FINISH

SND_SKIP

SPR
		STA WSYNC
		LDX #$40	; position the P0 sprite so that it's visible on the left
SPR.1	DEX
		BPL SPR.1
		STA RESP0

VBLANKWASTE
		LDA INTIM	; wait for end of VBLANK interval
		BNE VBLANKWASTE
		STA VBLANK	; turn off VBLANK
		
WASTESCREEN
		LDY #192
		LDX curStartCol
		;LDA v0jiff
		LDA v0mpos
		STA PF1
		LDA v0spos
		STA PF2
WASTESCREEN.1
		STA WSYNC

		LDA frameCnt	; player colors
		STA COLUP0
		LDA frameCnt+1
		STA COLUP1

		STX COLUBK		; background and playfield colors
		;STY PF1
		STY COLUPF
		
; ***** MORE BITS *****

		STY SWCHB		; is there RAM here?
		LDA SWCHB
		STA GRP0

; *****           *****
		INX
		DEY
		BNE WASTESCREEN.1
		
OVERSCAN
		STA WSYNC
		LDA #$02	; turn on VBLANK
		STA VBLANK
		LDA #$24	; set up timer for end of OVERSCAN
		STA TIM64T
OVERSCAN.1		
		LDA INTIM	; wait for end of OVERSCAN interval
		BNE OVERSCAN.1
		
		INC curStartCol
		
		JMP TOPSCAN
				
			org $FE80	; C scale in square, 0.3 format
S_LIST	.word S_TEST, S_MACD, S_MACE		
		
S_TEST	.word M_CSCT, M_CSCS, $80, $80	

S_MACE  .word M_DELAY
S_MACD	.word M_EIEIO, M_QUACK, M_EIEIO, $80, $80
		
M_CSCT	.byte $0C,$04,$01,$01	; timbres
		.byte $0F,$7F,$88,$7D,$88
		.byte $7A,$88,$77,$88,$F8
		.byte $0F,$73,$88,$71,$88,$6F,$88
		.byte $6E,$88,$F8,$0F,$6B,$88
		.byte $1F,$7F,$88,$7D,$88,$7A,$88
		.byte $77,$88,$F8
		.byte $1F,$73,$88,$71,$88,$6F,$88
		.byte $6E,$88,$F8,$1F,$6B,$88
		.byte $F8,$1F,$69,$88,$68,$88,$F8
		.byte $80
		
M_CSCS	.byte $0C,$04,$01,$01
		.byte $98,$2F,$7F,$88,$F8,$88
		.byte $2F,$77,$88,$F8,$2F,$72,$88
		.byte $F8,$2F,$6F,$88,$F8,$88
		.byte $2F,$67,$88,$F8,$88
		.byte $2F,$65,$88,$63,$88,$62,$88
		.byte $61,$88,$F8,$A8
		.byte $80

M_WALTZ	.byte $0C,$0C,$0C,$0C
		.byte $0C,$78,$8F,$F1
		.byte $0F,$6F,$8F,$F1
		.byte $0F,$6F,$8F,$F1
		.byte $0C,$7B,$8F,$F1
		.byte $0F,$6F,$8F,$F1
		.byte $0F,$6F,$8F,$F1
		.byte $80		
		
M_DELAY .byte $0C,$0C,$0C,$0C
		.byte $F1,$80
		
M_EIEIO .byte $0C,$04,$01,$01
		.byte $0C,$53,$88,$F8,$0C,$53,$88,$F8,$0C,$53,$88,$F8 ; old mac don
		.byte $0C,$5A,$88,$F8,$0C,$57,$88,$F8,$0C,$57,$88,$F8 ; ald had a
		.byte $0C,$5A,$98,$F8 ; farm
		.byte $0F,$4F,$8F,$F1,$0F,$4F,$8F,$F1 ; E I 
		.byte $0F,$51,$8F,$F1,$0F,$51,$8F,$F1 ; E I 
		.byte $0F,$53,$98,$F8 ; O-O-O
		.byte $F8,$08,$5A,$88,$F8 ; and / with a
		.byte $80
		
M_QUACK .byte $0C,$04,$01,$01
		.byte $3F,$47,$88,$F8,$3F,$47,$88,$F8,$0C,$53,$88,$F8 ; quack quack here
		.byte $04,$5A,$84,$F4,$04,$5A,$84,$F4 ; and a
		.byte $3F,$47,$88,$F8,$3F,$47,$88,$F8,$08,$53,$88,$F8 ; quack quack there
		.byte $90
		.byte $0C,$53,$84,$F4,$04,$53,$84,$F4,$38,$47,$84,$FC ; here a quack
		.byte $0C,$53,$84,$F4,$04,$53,$84,$F4,$38,$47,$84,$FC ; there a quack
		.byte $08,$53,$84,$F4,$04,$53,$84,$F4,$08,$53,$84,$F4 ; everywhere
		.byte $04,$53,$84,$F4,$3C,$47,$8C,$F4,$3C,$47,$8C,$F4 ; a quack quack
		.byte $84
		.byte $80

; note of five bits; timbre of four bits; volume of four bits
; lookup note, timbre, volume in table?  256 possible combinations
; combine timbre and volume -- lookup 3-bit timbre or 3-bit volume
; combine note and volume -- lookup 2-bit volume
; combine note and timbre -- lookup 2-bit timbre
; maximum delay of 256

; potential commands
; slur - no gap between notes (current default)
; gliss - incr/decr between notes (with delay factor)
; trill - always trill up
; vibrato - alternate between two volumes
; envelope - incr/decr between volumes (with delay factor)

; 0.3 format
; 00 - 0F: timbre #0 / volume
; 10 - 1F: timbre #1 / volume
; 20 - 2F: timbre #2 / volume
; 30 - 3F: timbre #3 / volume
; 40 - 7F: note or freq
; 80     : restart
; 81 - BF: delay next command in frames
; C0 - EF:
; F0     : restart
; F1 - FF: silence for 1-15 frames 

			org $FFFC
		.word INIT
		.word INIT
		